package com.jpattern.orm.test.query;

import static org.junit.Assert.assertEquals;

import java.sql.ResultSet;
import java.sql.SQLException;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;

import org.junit.Test;

import com.jpattern.orm.SharedBaseTest;
import com.jpattern.orm.IJPOrm;
import com.jpattern.orm.query.ISqlExecutor;
import com.jpattern.orm.session.IGeneratedKeyReader;
import com.jpattern.orm.session.IResultSetReader;
import com.jpattern.orm.session.ISession;
import com.jpattern.orm.transaction.ITransaction;

/**
 * 
 * @author Francesco Cina
 *
 * 02/lug/2011
 */
public class PlainSqlExecutorsTest extends SharedBaseTest {

	@Override
	protected void setUp() throws Exception {
	}

	@Override
	protected void tearDown() throws Exception {
	}

	@Test
	public void testExecuteAll() throws Exception {
		final IJPOrm jpOrm =getJPOrm();

		final ISession session = jpOrm.session();
		final ISqlExecutor sqlExecutor = session.sqlExecutor();

		ITransaction tx = session.beginTransaction();
		final List<Long> ids = sqlExecutorInsert( sqlExecutor );
		tx.commit();

		checkExistAll(ids , sqlExecutor , true);

		tx = session.beginTransaction();
		sqlExecutorDelete( ids, sqlExecutor );
		tx.commit();

		checkExistAll(ids , sqlExecutor , false);

	}


	private void sqlExecutorDelete(List<Long> ids, ISqlExecutor sqlExecutor) {
		final String sql = "delete from people where id = ?";

		final List<Object[]> args = new ArrayList<Object[]>();

		for (final Long id : ids) {
			args.add(new Object[]{id});
		}

		sqlExecutor.batchUpdate(sql, args).execute();

	}

	private List<Long> sqlExecutorInsert(ISqlExecutor sqlExec) {
		final List<Long> results = new ArrayList<Long>();

		long idMain = new Date().getTime();

		final long id1 = idMain++;
		results.add(id1);
		final String sqlFixed = "insert into people (id, firstname, lastname) values ( " + id1 + " , 'fixed name' , 'fixed surname' )";
		assertEquals( 1 , sqlExec.updateQuery(sqlFixed).update());

		final String sql1 = "insert into people (id, firstname, lastname) values ( ? , ? , ? )";
		final long id2 = idMain++;
		results.add(id2);
		assertEquals( 1 , sqlExec.updateQuery(sql1, new Object[]{ id2, "name-" + id2 , "surname-" + id2 }).update());

		final List<Object[]> args = new ArrayList<Object[]>();
		final long id3 = idMain++;
		final long id4 = idMain++;
		final long id5 = idMain++;
		results.add(id3);
		results.add(id4);
		results.add(id5);
		args.add( new Object[]{ id3, "name-" + id3 , "batchUpdate(sql1, args) " + id3 } );
		args.add( new Object[]{ id4, "name-" + id4 , "batchUpdate(sql1, args) " + id4 } );
		args.add( new Object[]{ id5, "name-" + id5 , "batchUpdate(sql1, args) " + id5 } );
		assertEquals( 3 , sqlExec.batchUpdate(sql1, args).update().length );

		final List<String> sqlsFixed = new ArrayList<String>();
		final long id6 = idMain++;
		final long id7 = idMain++;
		results.add(id6);
		results.add(id7);
		sqlsFixed.add("insert into people (id, firstname, lastname) values ( " + id6 + " , 'batchUpdate(sqlsFixed)' , '1' )");
		sqlsFixed.add("insert into people (id, firstname, lastname) values ( " + id7 + " , 'batchUpdate(sqlsFixed)' , '2' )");
		assertEquals( 2 , sqlExec.batchUpdate(sqlsFixed).update().length );

		final String sqlKeyExtractor = "insert into people (id, firstname, lastname) values ( SEQ_PEOPLE.nextval , ? , ? )";
		final IGeneratedKeyReader generatedKeyExtractor = new IGeneratedKeyReader() {
			/**
			 * 
			 */


			@Override
			public void read(ResultSet generatedKeyResultSet) throws SQLException {
				generatedKeyResultSet.next();
				final long gk = generatedKeyResultSet.getLong(1);
				System.out.println("Generated key: " + gk);
				results.add(gk);
			}

			@Override
			public String[] generatedColumnNames() {
				return new String[]{"ID"};
			}
		};
		assertEquals( 1 , sqlExec.updateQuery(sqlKeyExtractor, generatedKeyExtractor, new Object[]{ "sqlExec.update(sqlKeyExtractor, generatedKeyExtractor, args", "1"}).update() );

		return results;
	}

	private void checkExistAll(List<Long> peopleIds, ISqlExecutor sqlExecutor, boolean exist) {
		String sql = "select * from people where id in ( ";
		for (int i=0; i<peopleIds.size()-1; i++) {
			sql += "?, ";
		}
		sql += "? ) ";

		final IResultSetReader<List<Long>> rse = new IResultSetReader<List<Long>>() {
			/**
			 * 
			 */


			@Override
			public List<Long> read(ResultSet resultSet) throws SQLException {
				final List<Long> result = new ArrayList<Long>();
				while (resultSet.next()) {
					result.add( resultSet.getLong("ID") );
				}
				return result;
			}
		};

		final List<Long> result = sqlExecutor.findQuery(sql, peopleIds.toArray()).find(rse);
		for (final Long id : peopleIds) {
			System.out.println("Check id: " + id + " exists? " + result.contains(id) );
			assertEquals( exist,  result.contains(id) );
		}
	}
}
